# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "bruker"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&
    O|1|^1234567|
    R|1|A2-123456^1^NYME ANO^|STD|||A2^||
    L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&
    Q|1|^1010000007^|
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "1010000007"}],
    }
    assert worklist == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^&
    Q|1|^1010000007^|
    L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&
    L|1|N"""
    assert result == expected


def test_decode_result_with_example_protocol(self):
    date = datetime.datetime(2008, 3, 13, 14, 11, 45)
    worklist = self.decode(
        """
    H|\\^&
    P|1
    O|1|XY00012-1^Probe L^^2008-03-13T14:11:45^^^1810000007^A1
    R|1|^^^ORG^1|562^^A^^RUO^STD
    R|2|^^^PRO^1^1|562^Escherichia coli^+++^2.41^^BDAL
    R|3|^^^PRO^1^2|562^Escherichia coli^+++^2.34^^BDAL
    R|4|^^^PRO^1^3|562^Escherichia coli^+++^2.34^^BDAL
    R|5|^^^PRO^1^4|562^Escherichia coli^++^2.27^^BDAL
    R|6|^^^PRO^1^5|562^Escherichia coli^++^2.22^^BDAL
    R|7|^^^PRO^1^6|562^Escherichia coli^++^2.21^^BDAL
    R|8|^^^PRO^1^7|562^Escherichia coli^++^2.15^^BDAL
    R|9|^^^PRO^1^8|562^Escherichia coli^++^2.11^^BDAL
    R|10|^^^PRO^1^9|562^Escherichia coli^++^2.10^^BDAL
    R|11|^^^PRO^1^10|564^Escherichia fergusonii^+^1.93^^BDAL
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ORG", "date": date, "flags": ["A"], "value": "562"},
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["resultat actif", "2.41"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.34"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.34"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.27"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.22"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.21"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.15"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.11"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.10"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["1.93"],
                        "value": "564_Escherichia fergusonii",
                    },
                ],
                "sample_id": "1_1810000007_A",
                "type": "patient",
            }
        ],
    }

    assert worklist == expected


def test_decode_result_with_example_protocol_get_comment(self):
    date = datetime.datetime(2008, 3, 13, 14, 11, 45)
    worklist = self.decode(
        """
    H|\\^&
    P|1
    O|1|XY00012-1^Probe L^^2008-03-13T14:11:45^^^1810000007^A1
    R|1|^^^ORG^1|562^^A^^RUO^STD
    R|2|^^^PRO^1^1|562^Escherichia coli^+++^2.41^^BDAL
    R|3|^^^PRO^1^2|562^Escherichia coli^+++^2.34^^BDAL
    R|4|^^^PRO^1^3|562^Escherichia coli^+++^2.34^^BDAL
    R|5|^^^PRO^1^4|562^Escherichia coli^++^2.27^^BDAL
    R|6|^^^PRO^1^5|562^Escherichia coli^++^2.22^^BDAL
    R|7|^^^PRO^1^6|562^Escherichia coli^++^2.21^^BDAL
    R|8|^^^PRO^1^7|562^Escherichia coli^++^2.15^^BDAL
    R|9|^^^PRO^1^8|562^Escherichia coli^++^2.11^^BDAL
    R|10|^^^PRO^1^9|562^Escherichia coli^++^2.10^^BDAL
    R|11|^^^PRO^1^10|564^Escherichia fergusonii^+^1.93^^BDAL
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ORG", "date": date, "flags": ["A"], "value": "562"},
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["resultat actif", "2.41"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.34"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.34"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.27"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.22"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.21"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.15"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.11"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["2.10"],
                        "value": "562_Escherichia coli",
                    },
                    {
                        "code": "MALDI_A1",
                        "comments": [],
                        "date": date,
                        "flags": ["1.93"],
                        "value": "564_Escherichia fergusonii",
                    },
                ],
                "sample_id": "1_1810000007_A",
                "type": "patient",
            }
        ],
    }

    assert worklist == expected


def test_decode_result_with_real_example(self):
    date = datetime.datetime(2017, 9, 5, 17, 5, 25)
    worklist = self.decode(
        """
    H|\\^&
    P|1
    O|1|A2-TEST2^A2^^2017-09-05T17:05:25.699^^269944.00809^0020114226^A2
    R|1|^^^ORG^1|574^^A^^IVD^STD^
    R|2|^^^PRO^1^1|574^Klebsiella pneumoniae^+++^2.3^957e6d54-9e39-4411-b40a-aba10c9dd80e^IVD
    R|3|^^^PRO^1^2|72407^Klebsiella pneumoniae^++^2.28^a9d7959d-f118-4767-8c3d-45a1d78a55b6^IVD
    R|4|^^^PRO^1^3|573^Klebsiella pneumoniae^++^2.21^4a03d1e5-1c17-4b90-8d94-5091e2e7d7f8^IVD
    R|5|^^^PRO^1^4|72407^Klebsiella pneumoniae^++^2.2^3e02cabe-a9bd-4c66-bf2f-a615da9ba322^IVD
    R|6|^^^PRO^1^5|573^Klebsiella pneumoniae^++^2.17^64325e4d-299c-4140-9e74-b41e5df11188^IVD
    R|7|^^^PRO^1^6|573^Klebsiella pneumoniae^++^2.13^7e842d1d-d2e0-4346-9cfc-093c4642afda^IVD
    R|8|^^^PRO^1^7|72407^Klebsiella pneumoniae^++^2.09^feea4ab1-8d78-4d76-a3fa-8c3935cae865^IVD
    R|9|^^^PRO^1^8|574^Klebsiella pneumoniae^++^2.03^d0bae1a8-8c08-482c-81d7-eb39ae1d9bd0^IVD
    R|10|^^^PRO^1^9|244366^Klebsiella variicola^+^1.99^e6c028ed-e721-403d-a108-f966529dc6e3^IVD
    R|11|^^^PRO^1^10|244366^Klebsiella variicola^+^1.88^744bcac5-7fed-4c77-8394-e08d54eac6f5^IVD
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ORG", "date": date, "flags": ["A"], "value": "574"},
                    {
                        "code": "MALDI_A2",
                        "comments": ["957e6d54-9e39-4411-b40a-aba10c9dd80e"],
                        "date": date,
                        "flags": ["resultat actif", "2.3"],
                        "value": "574_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["a9d7959d-f118-4767-8c3d-45a1d78a55b6"],
                        "date": date,
                        "flags": ["2.28"],
                        "value": "72407_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["4a03d1e5-1c17-4b90-8d94-5091e2e7d7f8"],
                        "date": date,
                        "flags": ["2.21"],
                        "value": "573_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["3e02cabe-a9bd-4c66-bf2f-a615da9ba322"],
                        "date": date,
                        "flags": ["2.2"],
                        "value": "72407_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["64325e4d-299c-4140-9e74-b41e5df11188"],
                        "date": date,
                        "flags": ["2.17"],
                        "value": "573_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["7e842d1d-d2e0-4346-9cfc-093c4642afda"],
                        "date": date,
                        "flags": ["2.13"],
                        "value": "573_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["feea4ab1-8d78-4d76-a3fa-8c3935cae865"],
                        "date": date,
                        "flags": ["2.09"],
                        "value": "72407_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["d0bae1a8-8c08-482c-81d7-eb39ae1d9bd0"],
                        "date": date,
                        "flags": ["2.03"],
                        "value": "574_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["e6c028ed-e721-403d-a108-f966529dc6e3"],
                        "date": date,
                        "flags": ["1.99"],
                        "value": "244366_Klebsiella variicola",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["744bcac5-7fed-4c77-8394-e08d54eac6f5"],
                        "date": date,
                        "flags": ["1.88"],
                        "value": "244366_Klebsiella variicola",
                    },
                ],
                "sample_id": "TEST2_0020114226_A",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_real_example_get_comment(self):
    date = datetime.datetime(2017, 9, 5, 17, 5, 25)
    worklist = self.decode(
        """
    H|\\^&
    P|1
    O|1|A2-TEST2^A2^^2017-09-05T17:05:25.699^^269944.00809^0020114226^A2
    R|1|^^^ORG^1|574^^A^^IVD^STD^
    R|2|^^^PRO^1^1|574^Klebsiella pneumoniae^+++^2.3^957e6d54-9e39-4411-b40a-aba10c9dd80e^IVD
    R|3|^^^PRO^1^2|72407^Klebsiella pneumoniae^++^2.28^a9d7959d-f118-4767-8c3d-45a1d78a55b6^IVD
    R|4|^^^PRO^1^3|573^Klebsiella pneumoniae^++^2.21^4a03d1e5-1c17-4b90-8d94-5091e2e7d7f8^IVD
    R|5|^^^PRO^1^4|72407^Klebsiella pneumoniae^++^2.2^3e02cabe-a9bd-4c66-bf2f-a615da9ba322^IVD
    R|6|^^^PRO^1^5|573^Klebsiella pneumoniae^++^2.17^64325e4d-299c-4140-9e74-b41e5df11188^IVD
    R|7|^^^PRO^1^6|573^Klebsiella pneumoniae^++^2.13^7e842d1d-d2e0-4346-9cfc-093c4642afda^IVD
    R|8|^^^PRO^1^7|72407^Klebsiella pneumoniae^++^2.09^feea4ab1-8d78-4d76-a3fa-8c3935cae865^IVD
    R|9|^^^PRO^1^8|574^Klebsiella pneumoniae^++^2.03^d0bae1a8-8c08-482c-81d7-eb39ae1d9bd0^IVD
    R|10|^^^PRO^1^9|244366^Klebsiella variicola^+^1.99^e6c028ed-e721-403d-a108-f966529dc6e3^IVD
    R|11|^^^PRO^1^10|244366^Klebsiella variicola^+^1.88^744bcac5-7fed-4c77-8394-e08d54eac6f5^IVD
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ORG", "date": date, "flags": ["A"], "value": "574"},
                    {
                        "code": "MALDI_A2",
                        "comments": ["957e6d54-9e39-4411-b40a-aba10c9dd80e"],
                        "date": date,
                        "flags": ["resultat actif", "2.3"],
                        "value": "574_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["a9d7959d-f118-4767-8c3d-45a1d78a55b6"],
                        "date": date,
                        "flags": ["2.28"],
                        "value": "72407_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["4a03d1e5-1c17-4b90-8d94-5091e2e7d7f8"],
                        "date": date,
                        "flags": ["2.21"],
                        "value": "573_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["3e02cabe-a9bd-4c66-bf2f-a615da9ba322"],
                        "date": date,
                        "flags": ["2.2"],
                        "value": "72407_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["64325e4d-299c-4140-9e74-b41e5df11188"],
                        "date": date,
                        "flags": ["2.17"],
                        "value": "573_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["7e842d1d-d2e0-4346-9cfc-093c4642afda"],
                        "date": date,
                        "flags": ["2.13"],
                        "value": "573_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["feea4ab1-8d78-4d76-a3fa-8c3935cae865"],
                        "date": date,
                        "flags": ["2.09"],
                        "value": "72407_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["d0bae1a8-8c08-482c-81d7-eb39ae1d9bd0"],
                        "date": date,
                        "flags": ["2.03"],
                        "value": "574_Klebsiella pneumoniae",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["e6c028ed-e721-403d-a108-f966529dc6e3"],
                        "date": date,
                        "flags": ["1.99"],
                        "value": "244366_Klebsiella variicola",
                    },
                    {
                        "code": "MALDI_A2",
                        "comments": ["744bcac5-7fed-4c77-8394-e08d54eac6f5"],
                        "date": date,
                        "flags": ["1.88"],
                        "value": "244366_Klebsiella variicola",
                    },
                ],
                "sample_id": "TEST2_0020114226_A",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
