# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "cn3000_6000"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
    Q|1|000001^01^   123456^B||^^^040^PT\\^^^060^Fbg\\^^^120^II\\^^^150^V\\^^^170^VII\\^^^180^VIII\\^^^190^IX\\^^^200^X\\^^^210^XI\\^^^300^AT\\^^^310^APL|0|20110328133318
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "123456", "type": "patient"}],
    }
    assert result == expected


def test_decode_results(self):
    worklist = self.decode(
        """
    H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
    P|1||||^Heisei^Jiro
    O|1||000001^01^ 1^B^||R||||||N
    R|1|^^^041^PT sec^100.00^9^^^| 10.2|sec||N||||||20110328135056
    R|2|^^^042^PT %^100.00^9^^^| 99.4|%||N||||||20110328135056
    R|3|^^^043^PT R.^100.00^9^^^| 0.57|||N||||||20110328135056
    R|4|^^^044^PT INR^100.00^9^^^| 0.81|||N||||||20110328135056
    R|5|^^^051^APTT sec^100.00^9^^^| 27.4|sec||N||||||20110328135056
    R|6|^^^061^Fbg sec^100.00^9^^^| 8.5|sec||N||||||20110328135056
    R|7|^^^062^Fbg C.^100.00^9^^^| 588.2|mg/dL||N||||||20110328135056
    R|8|^^^^Hemolytic Sample^^^^^||||A||||||20110328135056
    R|9|^^^^Defective Sample Volume^^^^^||||N||||||20110328135056
    L|1|N
    """
    )
    date = datetime.datetime(2011, 3, 28, 13, 50, 56)
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "000001-01",
                "results": [
                    {
                        "code": "041",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "sec",
                        "value": "10.2",
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "042",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "%",
                        "value": "99.4",
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "043",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "",
                        "value": "0.57",
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "044",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "",
                        "value": "0.81",
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "051",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "sec",
                        "value": "27.4",
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "061",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "sec",
                        "value": "8.5",
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "062",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "mg/dL",
                        "value": "588.2",
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "Hemolytic Sample",
                        "date": date,
                        "dilution": "",
                        "unit": "",
                        "value": "YES",
                        "flags": ["A"],
                        "comments": [],
                    },
                    {
                        "code": "Defective Sample Volume",
                        "date": date,
                        "dilution": "",
                        "unit": "",
                        "value": "YES",
                        "flags": [],
                        "comments": [],
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
    P|1
    O|1||REAG00^  ^QCIHepLMW553322^M^||R||||||Q
    R|1|^^^342^INN Hep~IU_mL^100.00^A^^^|  0.41|IU/mL||N^^||||||20110328135407
    C|1|I|CAL^044^20110328^1^502501|I
    C|2|I|LOT^040^527501,527502|I
    C|3|I|QC^040^201103281354^^502701\\QC^040^201103281354^^512601|I
    L|1|N
    """
    )
    date = datetime.datetime(2011, 3, 28, 13, 54, 7)
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "502701, 512601",
                "results": [
                    {
                        "code": "342",
                        "date": date,
                        "dilution": "100.00",
                        "unit": "IU/mL",
                        "value": "0.41",
                        "flags": [],
                        "comments": [],
                    }
                ],
                "sample_id": "QCIHepLMW553322",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_results_with_comments(self):
    worklist = self.decode(
        """
    H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
    P|1||||^Heisei^Jiro
    O|1||000001^01^ 1^B^||R||||||N
    R|1|^^^041^PT sec^100.00^9^R^^| 10.2|sec||N||S||||20110328135407
    R|2|^^^042^PT %^100.00^9^R^^| 99.4|%||N||S||||20110328135407
    R|3|^^^043^PT R.^100.00^9^R^^| 0.57|||N||S||||20110328135407
    R|4|^^^044^PT INR^100.00^9^R^^| 0.81|||N||S||||20110328135407
    C|1|I|CAL^044^20110328^1^502501|I
    C|2|I|LOT^040^527501,527502|I
    C|3|I|QC^040^201103281354^^502701\\QC^040^201103281354^^512601|I
    L|1|N
    """
    )
    date = datetime.datetime(2011, 3, 28, 13, 54, 7)
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "000001-01",
                "results": [
                    {
                        "code": "041",
                        "date": date,
                        "dilution": "100.00",
                        "flags": [],
                        "unit": "sec",
                        "value": "10.2",
                        "comments": [],
                    },
                    {
                        "code": "042",
                        "date": date,
                        "dilution": "100.00",
                        "flags": [],
                        "unit": "%",
                        "value": "99.4",
                        "comments": [],
                    },
                    {
                        "code": "043",
                        "date": date,
                        "dilution": "100.00",
                        "flags": [],
                        "unit": "",
                        "value": "0.57",
                        "comments": [],
                    },
                    {
                        "code": "044",
                        "date": date,
                        "dilution": "100.00",
                        "flags": [],
                        "unit": "",
                        "value": "0.81",
                        "comments": [],
                        "lots": [
                            {"lot": "502501", "type": "calibration"},
                            {"lot": "527501", "type": "reagent"},
                            {"lot": "527502", "type": "reagent"},
                        ],
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_results_with_image_and_csv(self):
    worklist = self.decode(
        """
    H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
    P|1||||^Heisei^Jiro
    O|1||000001^01^ 1^B^||R||||||N
    R|1|^^^041^PT sec^100.00^9^^^| 10.2|sec||N||||||20110328135056
    R|2|^^^040^Normal^^^^^|PNG&R&20110328&R&2011_03_28_13_50_000001_040_Normal_100_1.PNG|||||||||20110328135056
    R|3|^^^050^ DelayedMixing | CSV&R&20110328&R&2011_03_28_13_50_000001_050_DelayedMixing.csv |||||||||20110328135056
    L|1|N
    """
    )
    date = datetime.datetime(2011, 3, 28, 13, 50, 56)
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "000001-01",
                "results": [
                    {
                        "code": "041",
                        "date": date,
                        "dilution": "100.00",
                        "flags": [],
                        "unit": "sec",
                        "value": "10.2",
                        "comments": [],
                    },
                    {
                        "code": "040",
                        "date": date,
                        "dilution": "",
                        "flags": [""],
                        "unit": "",
                        "value": "PNG/20110328/2011_03_28_13_50_000001_040_Normal_100_1.PNG",
                        "comments": [],
                    },
                    {
                        "code": "050",
                        "date": datetime.datetime(2011, 3, 28, 13, 50, 56),
                        "dilution": "",
                        "flags": [""],
                        "unit": "",
                        "value": "CSV/20110328/2011_03_28_13_50_000001_050_DelayedMixing.csv",
                        "comments": [],
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_results_with_flags(self):
    worklist = self.decode(
        """
    H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
    P|1||||^Heisei^Jiro
    O|1||000001^01^ 1^B^||R||||||N
    R|1|^^^041^PT sec^100.00^9^^^| 10.2|sec||<^[0000.6000.0200 Limite de rendu inférieure dépassée]^||||||20110328135056
    R|2|^^^042^PT %^100.00^9^^^| 99.4|%||>^[1100.4000.0000 Volume d'échantillon erroné],[0000.6000.0100 Limite de rendu supérieur dépassée]^||||||20110328135056
    R|3|^^^043^PT R.^100.00^9^^^|****.*|||A^^[34422 Reactif insuffisant]||||||20110328135056
    L|1|N
    """
    )
    date = datetime.datetime(2011, 3, 28, 13, 50, 56)
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "000001-01",
                "results": [
                    {
                        "code": "041",
                        "comments": [],
                        "date": date,
                        "dilution": "100.00",
                        "flags": ["<", "0000.6000.0200"],
                        "unit": "sec",
                        "value": "10.2",
                    },
                    {
                        "code": "042",
                        "comments": [],
                        "date": date,
                        "dilution": "100.00",
                        "flags": [">", "1100.4000.0000", "0000.6000.0100"],
                        "unit": "%",
                        "value": "99.4",
                    },
                    {
                        "code": "043",
                        "comments": ["34422 Reactif insuffisant"],
                        "date": date,
                        "dilution": "100.00",
                        "flags": ["A"],
                        "unit": "",
                        "value": "****.*",
                    },
                ],
                "sample_id": "1",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_query_order(self):
    worklist = self.encode(
        self.order_worklist,
        context={
            "query": self.parse(
                """
        H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
        Q|1|000001^01^         123456^B||^^^A^^1:10\\^^^B|0|20110328133318||||||C
        L|1|N
        """
            )
        },
    )
    expected = """
    H|\\^&|||||||||||E1394-97
    P|1|||036666666|^ANO^NYME
    O|1|000001^01^         123456^B||^^^A^^1:10\\^^^B|A||||||N
    L|1|N
    """
    assert worklist == expected


def test_void_order(self):
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
                H|\\^&|||CN-6000^00-03^10000001^^^CN-6000^XXXXXXXX||||||||E1394-97
                Q|1|000001^01^ 1^B||^^^A^^1:10\\^^^B|0|20110328133318
                L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&|||||||||||E1394-97
    P|1
    O|1|000001^01^              1^B
    L|1|N
    """
    assert result == expected
