# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy

parser_class = "dxc700"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].specimen = "W"
    message = self.order(
        worklist=worklist,
        context={"query": self.parse("R 99123412W1234                    123456")},
    )
    expected = "S 123412W1234          123456    EF001  036666666           NYME ANO            A  0B  0"
    assert message == expected


def test_decode_qc(self):
    worklist = self.decode(
        "DQ02      UQ001              U2 052E0352663C9562663D834                    20.49                            0272696B3522696B062                     3.55                            0322584152325841441                   494.67                            "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "U2",
                "results": [
                    {
                        "code": "035",
                        "flags": [],
                        "lots": [
                            {"lot": "2663", "type": "reagent"},
                            {"lot": "C956", "type": "reagent"},
                            {"lot": "2663", "type": "reagent"},
                            {"lot": "D834", "type": "reagent"},
                        ],
                        "value": "20.49",
                    },
                    {
                        "code": "027",
                        "flags": [],
                        "lots": [
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B352", "type": "reagent"},
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B062", "type": "reagent"},
                        ],
                        "value": "3.55",
                    },
                    {
                        "code": "032",
                        "flags": [],
                        "lots": [
                            {"lot": "2584", "type": "reagent"},
                            {"lot": "1523", "type": "reagent"},
                            {"lot": "2584", "type": "reagent"},
                            {"lot": "1441", "type": "reagent"},
                        ],
                        "value": "494.67",
                    },
                ],
                "sample_id": "U2",
                "specimen": "U",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        "D 02001002 0002    150000016001    EM0760049068               BREMENT Alain       0272696B3522696B062                     0.89                            02326244421                             2.19                            0282476831024767748                     0.52                            0342714268727142611                     1.31                            0492737D1772737D130                     2.35                            09626173722                          0  0  0                            "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0010-02",
                "results": [
                    {
                        "code": "027",
                        "flags": [],
                        "lots": [
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B352", "type": "reagent"},
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B062", "type": "reagent"},
                        ],
                        "value": "0.89",
                    },
                    {
                        "code": "023",
                        "flags": [],
                        "lots": [
                            {"lot": "2624", "type": "reagent"},
                            {"lot": "4421", "type": "reagent"},
                        ],
                        "value": "2.19",
                    },
                    {
                        "code": "028",
                        "flags": [],
                        "lots": [
                            {"lot": "2476", "type": "reagent"},
                            {"lot": "8310", "type": "reagent"},
                            {"lot": "2476", "type": "reagent"},
                            {"lot": "7748", "type": "reagent"},
                        ],
                        "value": "0.52",
                    },
                    {
                        "code": "034",
                        "flags": [],
                        "lots": [
                            {"lot": "2714", "type": "reagent"},
                            {"lot": "2687", "type": "reagent"},
                            {"lot": "2714", "type": "reagent"},
                            {"lot": "2611", "type": "reagent"},
                        ],
                        "value": "1.31",
                    },
                    {
                        "code": "049",
                        "flags": [],
                        "lots": [
                            {"lot": "2737", "type": "reagent"},
                            {"lot": "D177", "type": "reagent"},
                            {"lot": "2737", "type": "reagent"},
                            {"lot": "D130", "type": "reagent"},
                        ],
                        "value": "2.35",
                    },
                    {
                        "code": "096",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0  0  0",
                    },
                    {
                        "code": "L",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "I",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "H",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                ],
                "sample_id": "150000016001",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_hil_result(self):
    worklist = self.decode(
        "D 02001001 0013    150000015501    EF0020048879               Z_LABO Dï¿œborah    0272696B3522696B062                     0.98                            09626173722                          0  0  0                            "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0010-01",
                "results": [
                    {
                        "code": "027",
                        "flags": [],
                        "lots": [
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B352", "type": "reagent"},
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B062", "type": "reagent"},
                        ],
                        "value": "0.98",
                    },
                    {
                        "code": "096",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0  0  0",
                    },
                    {
                        "code": "L",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "I",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "H",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                ],
                "sample_id": "150000015501",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_iono_result(self):
    worklist = self.decode(
        "D 02000101 0001    150000006301    EF0020048870               Z_LABO Dï¿œborah    0972782    2742    2748                136.2G r     123032230332303423030982782    2742    2748                 4.61r       123032230332303423030992782    2742    2748                105.1r       12303223033230342303"
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0001-01",
                "results": [
                    {
                        "code": "097",
                        "flags": ["G", "r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "136.2",
                    },
                    {
                        "code": "098",
                        "flags": ["r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "4.61",
                    },
                    {
                        "code": "099",
                        "flags": ["r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "105.1",
                    },
                ],
                "sample_id": "150000006301",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_iono_qc(self):
    worklist = self.decode(
        "DQ02       Q001              L2 001E0972782    2742    2748                136.2G r     123032230332303423030982782    2742    2748                 4.61r       123032230332303423030992782    2742    2748                105.1r       12303223033230342303"
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "L2",
                "results": [
                    {
                        "code": "097",
                        "flags": ["G", "r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "136.2",
                    },
                    {
                        "code": "098",
                        "flags": ["r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "4.61",
                    },
                    {
                        "code": "099",
                        "flags": ["r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "105.1",
                    },
                ],
                "sample_id": "L2",
                "specimen": "S",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_complex_result(self):
    worklist = self.decode(
        "D 02000301 0001    150000129101    EM0270051486               Z_LABO DXC700 Margau0272695900326959102                     0.89r                           09626173722                          0  0  0r                           0972782    2742    2748                136.2G r     123032230332303423030982782    2742    2748                 4.61r       123032230332303423030992782    2742    2748                105.1r       12303223033230342303"
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0003-01",
                "results": [
                    {
                        "code": "027",
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2695", "type": "reagent"},
                            {"lot": "9003", "type": "reagent"},
                            {"lot": "2695", "type": "reagent"},
                            {"lot": "9102", "type": "reagent"},
                        ],
                        "value": "0.89",
                    },
                    {
                        "code": "096",
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0  0  0",
                    },
                    {
                        "code": "L",
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "I",
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "H",
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "097",
                        "flags": ["G", "r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "136.2",
                    },
                    {
                        "code": "098",
                        "flags": ["r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "4.61",
                    },
                    {
                        "code": "099",
                        "flags": ["r"],
                        "lots": [
                            {
                                "lot": "12303",
                                "name": "Na Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "22303",
                                "name": "K Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "32303",
                                "name": "Cl Electrode Lot No.",
                                "type": "reagent",
                            },
                            {
                                "lot": "42303",
                                "name": "REF Electrode Lot No.",
                                "type": "reagent",
                            },
                            {"lot": "2782", "type": "reagent"},
                            {"lot": "2742", "type": "reagent"},
                            {"lot": "2748", "type": "reagent"},
                        ],
                        "value": "105.1",
                    },
                ],
                "sample_id": "150000129101",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
