# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "gemweb"


def test_query(self):
    worklist = self.decode(
        """
    H|@^\\|||GEMweb Plus^1.0^Serveur GWP^863339^GWP.5.1.0.51.0.40.0|||||||P|LIS2-A|20171114103818
    Q|1|03032992^SAMPLE_NUMBER||||||||||D
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "03032992"}],
    }
    assert worklist == expected


def test_encode_order(self):
    message = self.order()
    expected = """
    H|@^\\||||||||||P|LIS2-A|20130715141912
    P|1||L036666666||NYME^ANO||20110513|F||||||123456^SAMPLE_NUMBER
    L|1
    """
    assert message == expected


def test_result(self):
    result = self.decode(
        """
    H|@^\\|||GEMweb Plus^1.0^Serveur GWP^863339^GWP.5.1.0.51.0.40.0|||||||P|LIS2-A|20171114104604
    P|1||900009349||ESSAI^Gazom^||19880705|M
    O|1|^03032992|0b2bbbdd-ac9a-4c2f-8f67-560c4d1dbff5||||^||150^uL||||||BLDA^^SY|||||||20171114104603|||F
    R|1|pH^^IL^pH|7.33||7.35 to 7.45^REFERENCE@7.15 to 7.6^CRITICAL|L||F||SUPERVISOR^^||20171114104037|G4K-TLAB2^T-LABO^GEM Premier 4000^14017137^181512058^A.3.1.99.0.31.0.0.24
    R|2|pCO2^^IL^pCO2|34|mm[Hg]|35.0 to 48.0^REFERENCE@20.0 to 50.0^CRITICAL|L||F||SUPERVISOR^^||20171114104037|G4K-TLAB2^T-LABO^GEM Premier 4000^14017137^181512058^A.3.1.99.0.31.0.0.24
    R|3|pO2^^IL^pO2|65|mm[Hg]|83.0 to 108.0^REFERENCE@50.0 to ^CRITICAL|L||F||SUPERVISOR^^||20171114104037|G4K-TLAB2^T-LABO^GEM Premier 4000^14017137^181512058^A.3.1.99.0.31.0.0.24
    R|4|Na+^^IL^Na+|137|mmol/L|136.0 to 145.0^REFERENCE@120.0 to 150.0^CRITICAL|N||F||SUPERVISOR^^||20171114104037|G4K-TLAB2^T-LABO^GEM Premier 4000^14017137^181512058^A.3.1.99.0.31.0.0.24
    R|5|O2ct^^IL^O2ct|12.0|mL/dL||||F||SUPERVISOR^^||20171114104037|G4K-TLAB2^T-LABO^GEM Premier 4000^14017137^181512058^A.3.1.99.0.31.0.0.24
    L|1|N"""
    )
    date = datetime.datetime(2017, 11, 14, 10, 40, 37)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "03032992",
                "results": [
                    {
                        "code": "pH",
                        "value": "7.33",
                        "flags": ["L"],
                        "operator": "SUPERVISOR",
                        "instrument": "G4K-TLAB2",
                        "date": date,
                        "lots": [{"lot": "181512058", "type": "reagent"}],
                    },
                    {
                        "code": "pCO2",
                        "value": "34",
                        "flags": ["L"],
                        "operator": "SUPERVISOR",
                        "instrument": "G4K-TLAB2",
                        "date": date,
                        "lots": [{"lot": "181512058", "type": "reagent"}],
                    },
                    {
                        "code": "pO2",
                        "value": "65",
                        "flags": ["L"],
                        "operator": "SUPERVISOR",
                        "instrument": "G4K-TLAB2",
                        "date": date,
                        "lots": [{"lot": "181512058", "type": "reagent"}],
                    },
                    {
                        "code": "Na+",
                        "value": "137",
                        "flags": ["N"],
                        "operator": "SUPERVISOR",
                        "instrument": "G4K-TLAB2",
                        "date": date,
                        "lots": [{"lot": "181512058", "type": "reagent"}],
                    },
                    {
                        "code": "O2ct",
                        "value": "12.0",
                        "flags": [],
                        "operator": "SUPERVISOR",
                        "instrument": "G4K-TLAB2",
                        "date": date,
                        "lots": [{"lot": "181512058", "type": "reagent"}],
                    },
                    {
                        "code": "OPE",
                        "value": "SUPERVISOR",
                        "instrument": "863339",
                        "date": date,
                    },
                    {
                        "code": "PRELEV",
                        "value": "BLDA",
                        "instrument": "863339",
                        "date": date,
                    },
                ],
            }
        ],
    }
    assert result == expected
