# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "infectio"


def test_encode_result_identification(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].specimen = "URINE"
    message = self.order(worklist=worklist, context={"control_id": "1521651964637"})
    expected = """
    MSH|^~\\&||INFECTIO||INFECTIO|20130715141912||OUL^R22|1521651964637|P|2.5|||NE|AL||UNICODE UTF-8
    PID|1||036666666||NYME^ANO||20110513|F|||||45 rue lambda\\S\\lala^31000^Toulouse
    PD1|1|||ATTENDING^LAGRANGE^Joseph-Louis
    PV1|1||^SER||||PRESCRIBER^Prescriber
    OBR|1|123456||Identification^Identification||20130603110342||||||||URINE|20110506121400
    OBX|1|CE|CultAerobie^Culture aerobie|1|1^1||||||F
    OBR|2|123456||Identification^Identification||20130603110342||||||||URINE|20110506121400
    OBX|1|CE|CultAerobie^Culture aerobie|2|2^2||||||F
    OBR|3|123456||B||20130603110342||||||||URINE|20110506121300
    OBX|1|DT|B||201302131516^201302131516||||||F
    """
    assert message == expected


def test_encode_result_atb(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].specimen = "URINE"
    worklist.samples[0].results = [
        {
            "code": "ECBU",
            "sub_code": "1",
            "value": "G_ECOLI",
            "date": datetime.datetime(2019, 5, 15, 12, 14),
            "atb": [
                {
                    "ANTIBIO": "A_A1",
                    "CMI": "3",
                    "SENSI": "S",
                    "DIAMETRE": "",
                    "CUTOFF": "",
                },
                {
                    "ANTIBIO": "A_A2",
                    "CMI": "4",
                    "SENSI": "I",
                    "DIAMETRE": "",
                    "CUTOFF": "",
                },
                {
                    "ANTIBIO": "A_A3",
                    "CMI": "5",
                    "SENSI": "R",
                    "DIAMETRE": "",
                    "CUTOFF": "",
                },
            ],
            "phenotypes": ["BLSE", "RM_CBPEN"],
        }
    ]
    message = self.order(worklist=worklist, context={"control_id": "1521651964637"})
    expected = """
    MSH|^~\\&||INFECTIO||INFECTIO|20130715141912||OUL^R22|1521651964637|P|2.5|||NE|AL||UNICODE UTF-8
    PID|1||036666666||NYME^ANO||20110513|F|||||45 rue lambda\\S\\lala^31000^Toulouse
    PD1|1|||ATTENDING^LAGRANGE^Joseph-Louis
    PV1|1||^SER||||PRESCRIBER^Prescriber
    OBR|1|123456||ECBU^ECBU||20130603110342||||||||URINE|20190515121400
    OBX|1|CE|CultAerobie^Culture aerobie|1|ECOLI||||||F
    OBX|2|ST|IDENT_PHENOECBU|1|BLSE||||||F
    OBX|3|ST|IDENT_PHENOECBU|1|RM_CBPEN||||||F
    OBR|2|123456||Antibiogramme^Antibiogramme||20130603110342||||||||URINE^1|20190515121400
    OBX|1|SN|A1||3|||S|||F|||20190515121400
    OBX|2|SN|A2||4|||I|||F|||20190515121400
    OBX|3|SN|A3||5|||R|||F|||20190515121400
    OBR|3|123456||ECBU^ECBU||20130603110342||||||||URINE|20190515121400
    OBX|1|CE|CultAerobie^Culture aerobie|1|ECOLI||||||F
    OBX|2|ST|IDENT_PHENOECBU|1|BLSE||||||F
    OBX|3|ST|IDENT_PHENOECBU|1|RM_CBPEN||||||F
    OBR|4|123456||Antibiogramme^Antibiogramme||20130603110342||||||||URINE^1|20190515121400
    OBX|1|SN|A1||3|||S|||F|||20190515121400
    OBX|2|SN|A2||4|||I|||F|||20190515121400
    OBX|3|SN|A3||5|||R|||F|||20190515121400
    OBR|5|123456||ECBU^ECBU||20130603110342||||||||URINE|20190515121400
    OBX|1|CE|CultAerobie^Culture aerobie|1|ECOLI||||||F
    OBX|2|ST|IDENT_PHENOECBU|1|BLSE||||||F
    OBX|3|ST|IDENT_PHENOECBU|1|RM_CBPEN||||||F
    OBR|6|123456||Antibiogramme^Antibiogramme||20130603110342||||||||URINE^1|20190515121400
    OBX|1|SN|A1||3|||S|||F|||20190515121400
    OBX|2|SN|A2||4|||I|||F|||20190515121400
    OBX|3|SN|A3||5|||R|||F|||20190515121400
    """
    assert message == expected
