# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "iq200xml"


@staticmethod
def unindent(data):
    return "".join(
        [li.lstrip().replace("\t", "") for li in data.splitlines() if li.strip()]
    )


def test_query(self):
    result = self.decode("""<SIQ>123456</SIQ>""")
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123456"}],
    }
    assert result == expected


def test_order(self):
    message = self.order()
    expected = """
    <?xml version="1.0" encoding="UTF-16"?><SI BF="URN" WO="run">123456<DEMOG DOB="2011-05-13" FName="ANO" Gender="F" LName="NYME" RecNum="036666666" /></SI>
        """
    assert message == expected


def test_decode_results(self):
    worklist = self.decode(
        """
    <SA BF="URN" ID="100007735165" SID="AP" OP="[auto]" ADT="2013-12-05 15:37:18-08:00" ADTS="2013-12-05 15:37:18" RDT="2013-12-05 15:37:53-08:00" RDTS="2013-12-05 15:37:53" RP="8" SQN="60100" RSQN="1" DILN="1" DILD="1" IMP="0" LName="LIORET" FName="Christiane" MName="[none]" DOB="1944-02-23" Loc="[none]" RecNum="03081700083352" Gender="F" REDT="2013-12-05 15:37:51-08:00" REDTS="2013-12-05 15:37:51">
        <PF>SELARL MIRIALIS</PF>
        <PF>509, Av Paul Bechet</PF>
        <PF>74300 Cluses</PF>
        <PF>numero serie IQ 5679</PF>
        <PF>Num Client: 6919</PF>
        <PF></PF>
        <PF></PF>
        <AC AT="Chemistry" AS="NotDone"/>
        <AC AT="Sediment" AS="Done" SO="Internal">
            <AR Key="RBC" SN="GR" LN="Globules rouges" AF="0" NR="10 /\xb5l">1 /5l</AR>
            <AR Key="WBC" SN="GB" LN="Globules blancs" AF="0" NR="10 /5l">9 /5l</AR>
            <AR Key="SQEP" SN="CELL EPI" LN="Cellules Epitheliales" AF="0" NR="[none]">QQS</AR>
            <AR Key="ASP" SN="CPP&lt;3um" LN="Compte de petites particules" AF="0" NR="5000 /5l">1256 /5l</AR>
            <AR Key="CAOX" SN="OXA DE CA" LN="Oxalate de calcium" AF="1" NR="RARES">QQS</AR>
            <AR Key="TYRO" SN="non utilisé" LN="non utilisé" AF="0" NR="PRÉSENT">[none]</AR>
            <AR Key="AMOR" SN="AMORPHE" LN="Cristal de Amorphes" AF="0" NR="RARES">FOO</AR>
            <AR Key="HYAL" SN="CYL HYALIN" LN="Cylindres hyalins" AF="0" NR="RARES">[none]</AR>
            <AR Key="WBCC" SN="Leucocytes" LN="Amas de leucocytes" AF="0" NR="RARES">QQS</AR>
            <AR Key="BYST" SN="Levures" LN="Levures" AF="0" NR="RARES">NBRS</AR>
        </AC>
        <FL>LIS T/O; CONCENTRATION ÉLEVÉE; ID D ÉCHANTILLON DOUBLE (NUMÉRO DE DOSSIER MÉDICAL DIFFÉRENT)</FL>
        <ARV>42</ARV>
        <AUD>
            <XE OP="[auto]" dt="2013-12-05 15:37:53"/>
        </AUD>
        <SysCfg MicroPresent="T" AXPresent="F" iChem200Present="F"/>
    </SA>"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "RBC", "dilution": "1", "value": "1"},
                    {"code": "WBC", "dilution": "1", "value": "9"},
                    {"code": "SQEP", "dilution": "1", "value": "QQS"},
                    {"code": "ASP", "dilution": "1", "value": "1256"},
                    {"code": "CAOX", "dilution": "1", "value": "QQS"},
                    {"code": "TYRO", "dilution": "1", "value": "[none]"},
                    {"code": "AMOR", "dilution": "1", "value": "FOO"},
                    {"code": "HYAL", "dilution": "1", "value": "[none]"},
                    {"code": "WBCC", "dilution": "1", "value": "QQS"},
                    {"code": "BYST", "dilution": "1", "value": "NBRS"},
                    {"code": "CYLINDRE", "dilution": "1", "value": "[none]"},
                    {"code": "CRISTAL", "dilution": "1", "value": "AMORFOO;CAOXQQS"},
                    {"code": "AUTRES", "dilution": "1", "value": "BYSTNBRS;WBCCQQS"},
                    {"code": "SEQUENCE", "value": "60100"},
                ],
                "sample_id": "100007735165",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_chemistry_results(self):
    worklist = self.decode(
        """
    <SA BF="URN" ID="00096264" SID="AP" OP="BB" ADT="2017-10-09 15:59:55+01:00" ADTS="2017-10-09 15:59:55" RDT="2017-10-09 16:31:46+01:00" RDTS="2017-10-09 16:31:46" RP="1" SQN="24649" RSQN="5" DILN="1" DILD="1" IMP="0" LName="ESSAI" FName="Admission" MName="[none]" DOB="1952-10-15" Loc="[none]" RecNum="18971" Gender="M" CDT="2017-10-09 16:02:25+01:00" CDTS="2017-10-09 16:02:25" REDT="2017-10-09 16:31:46+01:00" REDTS="2017-10-09 16:31:46"><PF>Centre Hospitalier de Bigorre</PF><PF>Avenue du Marechal de Lattre de Tassigny</PF>
        <PF>65000 TARBES</PF>
        <PF>IQ SN 8615 Velocity SN V03912</PF>
        <PF>NUMERO FAX : 01 49 90 91 27</PF>
        <PF></PF>
        <PF></PF>
        <AC AT="Chemistry" AS="Done" SO="External" DSLOT="7204160A">
            <AR Key="GLU" SN="GLU" LN="Glucose" AF="1" NR="TR">+++</AR>
            <AR Key="PRO" SN="PRO" LN="Protéine" AF="1" NR="TR">TR</AR>
            <AR Key="BIL" SN="BIL" LN="Bilirubine" AF="0" NR="+">NEG</AR>
            <AR Key="URO" SN="URO" LN="Urobilinogène" AF="0" NR="+">NEG</AR>
            <AR Key="PH" SN="pH" LN="pH" AF="0" NR="[none]">6,5</AR>
            <AR Key="BLD" SN="SNG" LN="Sang" AF="1" NR="TR">TR</AR>
            <AR Key="KET" SN="CET" LN="Cétones" AF="0" NR="TR">NEG</AR>
            <AR Key="NIT" SN="NIT" LN="Nitrite" AF="0" NR="+">NEG</AR>
            <AR Key="LEU" SN="LEU" LN="Leucocytes" AF="0" NR="TR">NEG</AR>
            <AR Key="CLA" SN="CLA" LN="Clarté" AF="0" NR="[none]">Clair</AR>
            <AR Key="SG" SN="D" LN="Densité" AF="0" NR="[none]">1,012</AR>
            <AR Key="COL" SN="COUL" LN="Couleur" AF="0" NR="[none]">Jaune clair</AR>
            <AR Key="CLA" SN="CLA" LN="Clarté" AF="0" NR="[none]">Légèrement brumeux</AR>
            <AR Key="ASA" SN="ASA" LN="Acide ascorbique" AF="0" NR="[none]">NEG</AR>
        </AC>
        <ARV>42</ARV>
        <AUD>
            <XE OP="BB" dt="2017-10-09 16:31:46"/><XE OP="VIM" dt="2017-10-09 16:03:55"><ED SN="POUBELLE" FROM="Aucun" TO="1 /µl"/><ED SN="HEMATIES" FROM="10 /µl" TO="8 /µl"/></XE><XE OP="[auto]" dt="2017-10-09 16:02:25"/>
        </AUD><SysCfg MicroPresent="T" AXPresent="F" iChem200Present="T"/>
        </SA>"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "GLU", "dilution": "1", "value": "+++"},
                    {"code": "PRO", "dilution": "1", "value": "TR"},
                    {"code": "BIL", "dilution": "1", "value": "NEG"},
                    {"code": "URO", "dilution": "1", "value": "NEG"},
                    {"code": "PH", "dilution": "1", "value": "6,5"},
                    {"code": "BLD", "dilution": "1", "value": "TR"},
                    {"code": "KET", "dilution": "1", "value": "NEG"},
                    {"code": "NIT", "dilution": "1", "value": "NEG"},
                    {"code": "LEU", "dilution": "1", "value": "NEG"},
                    {"code": "CLA", "dilution": "1", "value": "Clair"},
                    {"code": "SG", "dilution": "1", "value": "1,012"},
                    {"code": "COL", "dilution": "1", "value": "Jaune clair"},
                    {"code": "CLA", "dilution": "1", "value": "Légèrement brumeux"},
                    {"code": "ASA", "dilution": "1", "value": "NEG"},
                    {"code": "SEQUENCE", "value": "24649"},
                ],
                "sample_id": "00096264",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_internal_qc(self):
    date = datetime.datetime(2016, 1, 7, 11, 50, 56)
    worklist = self.decode(
        """
    <CQC CCType="+L" ID="FO15226G1131000000" SID="AP" OP="[auto]" ADT="2016-01-04 08:27:35+01:00" ADTS="2016-01-04 08:27:35" RDT="2016-01-07 11:50:56+01:00" RDTS="2016-01-07 11:50:56" LOT="226-2015-FO" EXPIR="03-2016" PRODID="FO" MEAN="1131" MNC="0" RP="6" SQN="0" RSQN="29">
        <PF>Labomaine</PF>
        <PF>38 rue de Guetteloup</PF>
        <PF>72000 Le Mans</PF>
        <PF>S/N 5596</PF>
        <PF>Cpte Client 1019</PF>
        <PF>Tel 02 43 78 40 60</PF>
        <PF></PF>
        <MIN>905</MIN>
        <MAX>1357</MAX>
        <COUNT>1122</COUNT>
        <STATUS>P</STATUS>
        <REF>2.087</REF>
    </CQC>"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "226-2015-FO",
                "results": [{"code": "FO", "date": date, "value": "1122"}],
                "sample_id": "226-2015-FO",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_external_qc(self):
    date = datetime.datetime(2017, 1, 19, 18, 50, 33)
    worklist = self.decode(
        """
    <SA BF="URN" ID="UR12161" SID="AP" OP="natalia" ADT="2017-01-19 18:44:09-08:00" ADTS="2017-01-19 18:44:09" RDT="2017-01-19 18:50:33-08:00" RDTS="2017-01-19 18:50:33" RP="1" SQN="393261" RSQN="4" DILN="1" DILD="1" IMP="0" REDT="2017-01-19 18:50:33-08:00" REDTS="2017-01-19 18:50:33"><PF>ANA-L</PF><PF>9 BOULEVARD DE VERDUN</PF><PF>94120 FONTENAY SOUS BOIS</PF><PF>IQ ELITE 6333</PF><PF>FAX BECKMAN COULTER 01 49 90 91 27</PF><PF></PF><PF></PF><AC AT="Chemistry" AS="NotDone"/><AC AT="Sediment" AS="Done" SO="Internal"><AR Key="ART" SN="ARTEFACT" LN="Artefact" AF="0" NR="99999999 /µl">[none]</AR><AR Key="RBC" SN="GR" LN="Globules rouges" AF="1" NR="10 /µl">17 /µl</AR><AR Key="DRBC" SN="GRD" LN="Globule rouge dysmorphique" AF="0" NR="[none]">[none]</AR><AR Key="WBC" SN="GB" LN="Globules blancs" AF="1" NR="10 /µl">31 /µl</AR><AR Key="WBCC" SN="AMAS GB" LN="Amas de globules blancs" AF="0" NR="RARES">[none]</AR><AR Key="BACT" SN="BACT&gt;3um" LN="Bactéries" AF="0" NR="PRESENCE">[none]</AR><AR Key="BYST" SN="LEVURES? GB?" LN="Levures" AF="0" NR="RARES">[none]</AR><AR Key="HYST" SN="MYCELLIUM" LN="Mycellium" AF="0" NR="RARES">[none]</AR><AR Key="SQEP" SN="CELL EPI" LN="Cellules Epitheliales" AF="0" NR="[none]">[none]</AR><AR Key="TREP" SN="non utilisé" LN="non utilisé" AF="0" NR="6 /µl">[none]</AR><AR Key="REEP" SN="non utilisé" LN="non utilisé" AF="0" NR="6 /µl">[none]</AR><AR Key="OVFB" SN="non utilisé" LN="non utilisé" AF="0" NR="1 /µl">[none]</AR><AR Key="FAT" SN="non utilisé" LN="non utilisé" AF="0" NR="1 /µl">[none]</AR><AR Key="MUCS" SN="MUCUS" LN="Mucus" AF="0" NR="+-">[none]</AR><AR Key="RBCC" SN="non utilisé" LN="non utilisé" AF="0" NR="RARE">[none]</AR><AR Key="SPRM" SN="SPERMATO" LN="Spermatozoïdes" AF="0" NR="QQS">[none]</AR><AR Key="TRCH" SN="TRICHO" LN="Trichomonas" AF="0" NR="PRESENCE">[none]</AR><AR Key="NSE" SN="PTES CELL RDES" LN="Petites cellules rondes" AF="0" NR="RARES">[none]</AR><AR Key="UNCC" SN="CYL NON ID" LN="Cylindres Pathologiques" AF="0" NR="RARES">[none]</AR><AR Key="HYAL" SN="CYL HYALIN" LN="Cylindre d'hyalin" AF="0" NR="RARES">[none]</AR><AR Key="EPIC" SN="non utilisé" LN="non utilisé" AF="0" NR="0 /µl">[none]</AR><AR Key="WBCT" SN="CYL LEUCO" LN="Cylindres Leucocytaires" AF="0" NR="RARES">[none]</AR><AR Key="RBCT" SN="CYL HEMA" LN="Cylindres Hematiques" AF="0" NR="RARES">[none]</AR><AR Key="GRAN" SN="CYL GRAN" LN="Cylindres granuleux" AF="0" NR="RARES">[none]</AR><AR Key="CELL" SN="non utilise" LN="non utilisé" AF="0" NR="rares">[none]</AR><AR Key="BROAD" SN="non utilisé" LN="non utilisé" AF="0" NR="1 /FaG">[none]</AR><AR Key="FATC" SN="non utilisé" LN="non utilisé" AF="0" NR="1 /FaG">[none]</AR><AR Key="WAXY" SN="non utilise" LN="non utilise" AF="0" NR="1 /µl">[none]</AR><AR Key="UNCX" SN="CRTX NON ID" LN="Cristaux non identifiées" AF="0" NR="RARES">[none]</AR><AR Key="TPO4" SN="P.A.M" LN="Phosphates ammoniaco magnesiens" AF="0" NR="QQS">[none]</AR><AR Key="CAOX" SN="OXA DE CA" LN="Oxalate de calcium" AF="0" NR="RARES">[none]</AR><AR Key="CAPH" SN="PHOS DE CA" LN="Phosphate de calcium" AF="0" NR="RARES">[none]</AR><AR Key="LEUC" SN="LEUC" LN="Cristal de leucine" AF="0" NR="PRESENCE">[none]</AR><AR Key="CYST" SN="CYST" LN="Cristal de cystine" AF="0" NR="[none]">[none]</AR><AR Key="TYRO" SN="TYRO" LN="Cristal de tyrosine" AF="0" NR="PRESENCE">[none]</AR><AR Key="AMOR" SN="AMOR" LN="Amorphe" AF="0" NR="RARES">[none]</AR><AR Key="UNCL" SN="Vue globale" LN="Vue globale" AF="0" NR="99999999 /µl">[none]</AR><AR Key="PC" SN="Ptes Particules &lt;3um" LN="Compte de petites particules" AF="0" NR="7500 /µl">3339 /µl</AR></AC><FL>LIS T/O</FL><ARV>42</ARV><AUD><XE OP="natalia" dt="2017-01-19 18:50:33"/><XE OP="natalia" dt="2017-01-19 18:50:29" EdFL="T"/></AUD><SysCfg MicroPresent="T" AXPresent="F" iChem200Present="F"/></SA>"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "UR12161",
                "results": [
                    {"code": "RBC", "date": date, "value": "17"},
                    {"code": "WBC", "date": date, "value": "31"},
                    {"code": "PC", "date": date, "value": "3339"},
                    {"code": "CYLINDRE", "date": date, "value": "[none]"},
                    {"code": "CRISTAL", "date": date, "value": "[none]"},
                ],
                "sample_id": "UR12161",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_not_external(self):
    self.configure(iq200xml_external_qc=False)
    worklist = self.decode(
        """
    <SA BF="URN" ID="UR100007735165" SID="AP" OP="[auto]" ADT="2013-12-05 15:37:18-08:00" ADTS="2013-12-05 15:37:18" RDT="2013-12-05 15:37:53-08:00" RDTS="2013-12-05 15:37:53" RP="8" SQN="60100" RSQN="1" DILN="1" DILD="1" IMP="0" LName="LIORET" FName="Christiane" MName="[none]" DOB="1944-02-23" Loc="[none]" RecNum="03081700083352" Gender="F" REDT="2013-12-05 15:37:51-08:00" REDTS="2013-12-05 15:37:51">
        <PF>SELARL MIRIALIS</PF>
        <PF>509, Av Paul Bechet</PF>
        <PF>74300 Cluses</PF>
        <PF>numero serie IQ 5679</PF>
        <PF>Num Client: 6919</PF>
        <PF></PF>
        <PF></PF>
        <AC AT="Chemistry" AS="NotDone"/>
        <AC AT="Sediment" AS="Done" SO="Internal">
            <AR Key="RBC" SN="GR" LN="Globules rouges" AF="0" NR="10 /\xb5l">1 /5l</AR>
            <AR Key="WBC" SN="GB" LN="Globules blancs" AF="0" NR="10 /5l">9 /5l</AR>
            <AR Key="SQEP" SN="CELL EPI" LN="Cellules Epitheliales" AF="0" NR="[none]">QQS</AR>
            <AR Key="ASP" SN="CPP&lt;3um" LN="Compte de petites particules" AF="0" NR="5000 /5l">1256 /5l</AR>
            <AR Key="CAOX" SN="OXA DE CA" LN="Oxalate de calcium" AF="1" NR="RARES">QQS</AR>
            <AR Key="TYRO" SN="non utilisé" LN="non utilisé" AF="0" NR="PRÉSENT">[none]</AR>
            <AR Key="AMOR" SN="AMORPHE" LN="Cristal de Amorphes" AF="0" NR="RARES">FOO</AR>
            <AR Key="HYAL" SN="CYL HYALIN" LN="Cylindres hyalins" AF="0" NR="RARES">[none]</AR>
            <AR Key="WBCC" SN="Leucocytes" LN="Amas de leucocytes" AF="0" NR="RARES">QQS</AR>
            <AR Key="BYST" SN="Levures" LN="Levures" AF="0" NR="RARES">NBRS</AR>
        </AC>
        <FL>LIS T/O; CONCENTRATION ÉLEVÉE; ID D ÉCHANTILLON DOUBLE (NUMÉRO DE DOSSIER MÉDICAL DIFFÉRENT)</FL>
        <ARV>42</ARV>
        <AUD>
            <XE OP="[auto]" dt="2013-12-05 15:37:53"/>
        </AUD>
        <SysCfg MicroPresent="T" AXPresent="F" iChem200Present="F"/>
    </SA>"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "RBC", "dilution": "1", "value": "1"},
                    {"code": "WBC", "dilution": "1", "value": "9"},
                    {"code": "SQEP", "dilution": "1", "value": "QQS"},
                    {"code": "ASP", "dilution": "1", "value": "1256"},
                    {"code": "CAOX", "dilution": "1", "value": "QQS"},
                    {"code": "TYRO", "dilution": "1", "value": "[none]"},
                    {"code": "AMOR", "dilution": "1", "value": "FOO"},
                    {"code": "HYAL", "dilution": "1", "value": "[none]"},
                    {"code": "WBCC", "dilution": "1", "value": "QQS"},
                    {"code": "BYST", "dilution": "1", "value": "NBRS"},
                    {"code": "CYLINDRE", "dilution": "1", "value": "[none]"},
                    {"code": "CRISTAL", "dilution": "1", "value": "AMORFOO;CAOXQQS"},
                    {"code": "AUTRES", "dilution": "1", "value": "BYSTNBRS;WBCCQQS"},
                    {"code": "SEQUENCE", "value": "60100"},
                ],
                "sample_id": "UR100007735165",
                "type": "patient",
            }
        ],
    }

    assert worklist == expected


def test_decode_result_option_cylindres_cristaux_autres(self):
    self.configure(
        iq200xml_cristaux="CAOX;TPO4;URIC;AMOR;CAPH;CACB;LEUC;CYST;TYRO;UNCX",
        iq200xml_cylindres="HYAL;GRAN;RBCT;WBCT;UNCC",
        iq200xml_autres="WBCC;BACT;BYST;SQEP;NSE;SPRM;TRCH;MUCS",
    )
    worklist = self.decode(
        """
    <SA BF="URN" ID="110000924458" SID="AP" OP="RESP" ADT="2022-12-14 17:06:42+01:00" ADTS="2022-12-14 17:06:42" RDT="2022-12-14 17:46:51+01:00" RDTS="2022-12-14 17:46:51" RP="1" SQN="462752" RSQN="9" DILN="1" DILD="1" IMP="0" LName="Z_LABO" FName="Jordan" MName="[none]" DOB="2000-01-01" Loc="[none]" RecNum="30681" Gender="F" REDT="2022-12-14 17:46:51+01:00" REDTS="2022-12-14 17:46:51">
        <PF>numero serie IQ C15561</PF>
        <PF></PF>
        <PF></PF>
        <PF></PF>
        <PF></PF>
        <PF></PF>
        <PF></PF>
        <AC AT="Chemistry" AS="NotDone"/>
        <AC AT="Sediment" AS="Done" SO="Internal">
            <AR Key="RBC" SN="GR" LN="Globules rouges" AF="0" NR="10 /µl">0</AR>
            <AR Key="WBC" SN="GB" LN="Globules blancs" AF="1" NR="10 /µl">15 /µl</AR>
            <AR Key="WBCC" SN="AMAS GB" LN="Amas de globules blancs" AF="0" NR="PRESENCE">PRESENCE</AR>
            <AR Key="UNCC" SN="Cylindres patho" LN="Cylindres Pathologiques" AF="1" NR="PRESENCE">PRESENCE</AR>
            <AR Key="HYAL" SN="CYL HYALIN" LN="Cylindres hyalins" AF="0" NR="PRESENCE">[none]</AR>
            <AR Key="WBCT" SN="non utilise" LN="non utilise" AF="0" NR="PRESENCE">QQS</AR>
            <AR Key="UNCX" SN="CRSTX NON ID" LN="Cristaux non identifiés" AF="0" NR="PRESENCE">[none]</AR>
            <AR Key="TPO4" SN="P.A.M" LN="Phosphate amoniaco-magnesien" AF="1" NR="PRESENCE">PRESENCE</AR>
            <AR Key="CAOX" SN="OXA DE CA" LN="Oxalate de calcium" AF="0" NR="PRESENCE">QQS</AR>
            <AR Key="UNCL" SN="Vue Global" LN="Vue Global" AF="0" NR="99999999 /µl">[none]</AR>
        </AC>
        <ARV>42</ARV>
        <AUD>
            <XE OP="RESP" dt="2022-12-14 17:46:51"><ED SN="P.A.M" FROM="Aucun" TO="PRESENCE"/><ED SN="AMORPHES" FROM="PRESENCE" TO="Aucun"/></XE><XE OP="RESP" dt="2022-12-14 17:38:23"/><XE OP="RESP" dt="2022-12-14 17:19:18"/><XE OP="RESP" dt="2022-12-14 17:09:50"/>
        </AUD>
        <SysCfg MicroPresent="T" AXPresent="F" iChem200Present="F"/>
    </SA>
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "RBC", "dilution": "1", "value": "0"},
                    {"code": "WBC", "dilution": "1", "value": "15"},
                    {"code": "WBCC", "dilution": "1", "value": "PRESENCE"},
                    {"code": "UNCC", "dilution": "1", "value": "PRESENCE"},
                    {"code": "HYAL", "dilution": "1", "value": "[none]"},
                    {"code": "WBCT", "dilution": "1", "value": "QQS"},
                    {"code": "UNCX", "dilution": "1", "value": "[none]"},
                    {"code": "TPO4", "dilution": "1", "value": "PRESENCE"},
                    {"code": "CAOX", "dilution": "1", "value": "QQS"},
                    {"code": "UNCL", "dilution": "1", "value": "[none]"},
                    {
                        "code": "CYLINDRE",
                        "dilution": "1",
                        "value": "UNCCPRESENCE;WBCTQQS",
                    },
                    {
                        "code": "CRISTAL",
                        "dilution": "1",
                        "value": "CAOXQQS;TPO4PRESENCE",
                    },
                    {"code": "AUTRES", "dilution": "1", "value": "WBCCPRESENCE"},
                    {"code": "SEQUENCE", "value": "462752"},
                ],
                "sample_id": "110000924458",
                "type": "patient",
            }
        ],
    }

    assert worklist == expected
