# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "isys"


def test_order(self):
    result = self.order()
    expected = """H|\\^_|||MCA|||||||P|2|20130715141912
        P|1||036666666||NYME^ANO||20110513|F
        O|1|^123456||^^^A\\^^^B|S||||||A||||||||||||||O
        O|2|^123456||^^^C|S||||||C||||||||||||||O
        L|1|N"""
    assert result == expected


def test_query(self):
    result = self.decode(
        """
    H|\\^_|||IDS-iSYS B300B0357|||||||P|4|20131218163740
    Q|1|^593512188879||ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "593512188879", "type": "patient"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^_|||IDS-iSYS B300B0357|||||||P|4|20131218163740
    Q|1|^593512188879||ALL||||||||O
    L|1|N"""
            )
        },
    )
    expected = """
    H|\\^_|||MCA|||||||P|2|20130715141912
    P|1
    O|1|^593512188879|||||||||||||||||||||||Y
    L|1|N"""
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^_|||IDS-iSYS B300B0357|||||||P|4|20131218172725
    P|1||||CTX-ICTL3|||||||||02
    O|1|215826401215||^^^CTX-I^02|S||||||||||S
    R|1|^^^CTX-I^02^F^1^23|2,067|ng/mL||N||F||||20131218172725|IDS-iSYS B300B0357
    M|1|^^^CTX-I^02^F^1^23|51|2,05^1,68^2,42^1582^^20140731|CTX-I^1670^216706800684^20140731|System Liquid^76206^00176^20150531\\Cuvettes^22311^01338^20160327|TRIGB^76204^00499^20150531\\TRIGA^76203^00452^20150531\\Wash S^76208^00130^20150531\\Wash1^1\\Wash2\\Wash3\\DSORB
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "1582",
                "control_name": "CTX-ICTL3",
                "control_lot": "1582",
                "results": [
                    {
                        "flags": [],
                        "comments": [],
                        "code": "CTX-I",
                        "value": "2.067",
                        "instrument": "IDS-iSYS B300B0357",
                        "date": datetime.datetime(2013, 12, 18, 17, 27, 25),
                        "lots": [
                            {
                                "expiration": datetime.datetime(2014, 7, 31, 0, 0),
                                "lot": "1670",
                                "name": "1670",
                                "type": "control",
                            }
                        ],
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^_|||IDS-iSYS B300B0357|||||||P|4|20131218171753
    P|1||01130925250922||ANO^NYME||19420419|M
    O|1|593512188879||^^^CTX-I|R||||||||||O
    R|1|^^^CTX-I^04^F^1|<0,033|ng/mL||<||F||||20131218171753|IDS-iSYS B300B0357
    C|1|I|HDM-|N
    M|1|^^^CTX-I^04^F^1|51||CTX-I^1670^216706800684^20140731|System Liquid^76206^00176^20150531\\Cuvettes^22311^01338^20160327|TRIGB^76204^00499^20150531\\TRIGA^76203^00452^20150531\\Wash S^76208^00130^20150531\\Wash1^2\\Wash2\\Wash3\\DSORB
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "593512188879",
                "results": [
                    {
                        "flags": ["<"],
                        "instrument": "IDS-iSYS B300B0357",
                        "comments": ["HDM-"],
                        "date": datetime.datetime(2013, 12, 18, 17, 17, 53),
                        "lots": [
                            {
                                "expiration": datetime.datetime(2014, 7, 31, 0, 0),
                                "lot": "1670",
                                "name": "1670",
                                "type": "reagent",
                            }
                        ],
                        "value": "<0,033",
                        "code": "CTX-I",
                    }
                ],
            }
        ],
    }
    assert result == expected
