# -*- coding: utf8 -*-
from __future__ import unicode_literals

import pytest

parser_class = "navios"


def test_decode_result(self):
    self.configure(navios_identification_qc="IMMUNOTROL;")
    # un \t en trop : pas de résultat en trop
    worklist = self.decode(
        """
Navios Cytometer Wednesday, September 23, 2020 09:05:07
File                                                               	Gate                                	X Param  	Y Param	[@BARCODE]	Region  	Abs Count	%Gated	resu4	resu5
7587191 IMMUNOTROL NOR 7587191 DPH 00027654 2020-09-23 0905 001.LMD	ADJLIN1 [GADJ AND LADJ AND ADJLIN 1]	CD8-ECD  	CD4-RD1	7587191   	ALL     	ERROR    	100.00	4.0   	5.00 
"""  # noqa
    )
    assert worklist == {
        "samples": [
            {
                "control_lot": "7587191",
                "sample_id": "7587191",
                "type": "control",
                "results": [
                    {"code": "ALLAbs Count", "value": "ERROR"},
                    {"code": "ALL%Gated", "value": "100.00"},
                    {"code": "ALLresu4", "value": "4.0"},
                    {"code": "ALLresu5", "value": "5.00"},
                ],
            }
        ],
        "context": "result",
    }

    worklist = self.decode(
        """


    Navios Cytometer Wednesday, June 20, 2018 14:24:27
    File                                                                         	Gate                	X Param   	Y Param   	[@BARCODE]	Region	Abs Count	Number	%Gated
    2405221697 CATOIRE THIERRY 2405221697 ELODIE 00011407 2018-05-23 1413 002.LM	B [CD45 AND B]      	CD45 FITC 	SS INT LIN	2405221697	ALL   	17       	35    	100.00
    2405221697 CATOIRE THIERRY 2405221697 ELODIE 00011407 2018-05-23 1413 002.LMD	C [CD45 AND B AND C]	FS INT LIN	SS INT LIN	2405221697	ALL   	2        	5     	100.00
    2405221697 CATOIRE THIERRY 2405221697 ELODIE 00011407 2018-05-23 1413 002.LMD	CD45 [CD45]         	CD34 PE   	SS INT LIN	2405221697	B     	17       	35    	0.31
    2405221697 CATOIRE THIERRY 2405221697 ELODIE 00011407 2018-05-23 1413 002.LMD	B [CD45 AND B]      	CD45 FITC 	SS INT LIN	2405221697	C     	2        	5     	14.29
    2405221697 CATOIRE THIERRY 2405221697 ELODIE 00011407 2018-05-23 1413 002.LMD	C [CD45 AND B AND C]	FS INT LIN	SS INT LIN	2405221697	C34VA 	2        	5     	100.00
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ALLAbs Count", "value": "17"},
                    {"code": "ALLNumber", "value": "35"},
                    {"code": "ALL%Gated", "value": "100.00"},
                    {"code": "ALLAbs Count", "value": "2"},
                    {"code": "ALLNumber", "value": "5"},
                    {"code": "ALL%Gated", "value": "100.00"},
                    {"code": "BAbs Count", "value": "17"},
                    {"code": "BNumber", "value": "35"},
                    {"code": "B%Gated", "value": "0.31"},
                    {"code": "CAbs Count", "value": "2"},
                    {"code": "CNumber", "value": "5"},
                    {"code": "C%Gated", "value": "14.29"},
                    {"code": "C34VAAbs Count", "value": "2"},
                    {"code": "C34VANumber", "value": "5"},
                    {"code": "C34VA%Gated", "value": "100.00"},
                ],
                "sample_id": "2405221697",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_without_column_number(self):
    worklist = self.decode(
        """


    Navios Cytometer Wednesday, June 20, 2018 14:28:27
    File                                                                         	Gate                             	X Param  	Y Param 	[@BARCODE]	Region    	Abs Count	%Gated
    2406191595 MORETTI MANUEL 2406191595 CORINNE 00011905 2018-06-20 1054 001.LMD	ADJ- [GADJ AND LADJ AND ADJ-]    	CD56-RD1 	CD16-PC7	2406191595	A1        	ERROR    	0.23
    2406191595 MORETTI MANUEL 2406191595 CORINNE 00011905 2018-06-20 1054 001.LMD	ADJ- [GADJ AND LADJ AND ADJ-]    	CD56-RD1 	CD16-PC7	2406191595	A3        	ERROR    	43.57
    2406191595 MORETTI MANUEL 2406191595 CORINNE 00011905 2018-06-20 1054 001.LMD	ADJ- [GADJ AND LADJ AND ADJ-]    	CD56-RD1 	CD16-PC7	2406191595	A4        	ERROR    	44.92
    2406191595 MORETTI MANUEL 2406191595 CORINNE 00011905 2018-06-20 1054 001.LMD	LADJ [GADJ AND LADJ]             	CD3-PC5  	        	2406191595	ADJ-      	ERROR    	26.04
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "A1Abs Count", "value": "ERROR"},
                    {"code": "A1%Gated", "value": "0.23"},
                    {"code": "A3Abs Count", "value": "ERROR"},
                    {"code": "A3%Gated", "value": "43.57"},
                    {"code": "A4Abs Count", "value": "ERROR"},
                    {"code": "A4%Gated", "value": "44.92"},
                    {"code": "ADJ-Abs Count", "value": "ERROR"},
                    {"code": "ADJ-%Gated", "value": "26.04"},
                ],
                "sample_id": "2406191595",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_manual(self):
    worklist = self.decode(
        """


    Navios Cytometer Wednesday, September 04, 2019 12:22:37
    File                                                                         	Gate                             	X Param  	Y Param 	[@BARCODE]	Region    	Abs Count	%Gated
    ESSAI ESSAI ESSAI 2409041364 SABRINA 00020737 2019-09-04 1216 002.LMD
    \tB[CD45 AND B]         \tCD45 FITC   \tSS INT LIN  \t2409041364  \tALL       \t174         \t100.00
    ESSAI ESSAI ESSAI 2409041364 SABRINA 00020737 2019-09-04 1216 002.LMD  \tC[CD45 AND B AND C]   \tFS INT LIN \tSS INT LIN  \t2409041364  \tALL       \t43          \t100.00
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


@pytest.mark.parametrize("navios_identification_qc", ["IMMUNOTROL;QC", "IMMUNOTROL"])
def test_decode_control(self, navios_identification_qc):
    self.configure(navios_identification_qc=navios_identification_qc)
    worklist = self.decode(
        """

    Navios Cytometer Tuesday, September 03, 2019 17:03:28
    File                                                                   	Gate         	X Param   	Y Param   	[@BARCODE]	Region   	Abs Count	%Gated
    7609234 IMMUNOTROL LOW 7609234 SABRINA 00020711 2019-09-03 1415 001.LMD	LYMPH [LYMPH]	CD19 FITC 	          	7609234   	ALL      	ERROR    	100.00
    7609234 IMMUNOTROL LOW 7609234 SABRINA 00020711 2019-09-03 1415 001.LMD	LYMPH [LYMPH]	CD19 FITC 	CD5 PE    	7609234   	ALL      	ERROR    	100.00
    7609234 IMMUNOTROL LOW 7609234 SABRINA 00020711 2019-09-03 1415 001.LMD	LYMPH [LYMPH]	CD5 PE    	          	7609234   	ALL      	ERROR    	100.00
    7609234 IMMUNOTROL LOW 7609234 SABRINA 00020711 2019-09-03 1415 001.LMD	Ungated      	SS INT LIN	FS INT LIN	7609234   	ALL      	ERROR    	100.00
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ALLAbs Count", "value": "ERROR"},
                    {"code": "ALL%Gated", "value": "100.00"},
                    {"code": "ALLAbs Count", "value": "ERROR"},
                    {"code": "ALL%Gated", "value": "100.00"},
                    {"code": "ALLAbs Count", "value": "ERROR"},
                    {"code": "ALL%Gated", "value": "100.00"},
                    {"code": "ALLAbs Count", "value": "ERROR"},
                    {"code": "ALL%Gated", "value": "100.00"},
                ],
                "sample_id": "7609234",
                "control_lot": "7609234",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
