# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "tecan_evolution"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||P||20130715141912
    P|1|036666666|036666666|036666666
    O|1|123456||^^^A^1:10\\^^^B
    L|1|N"""
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||TECAN EVOlution^V 2.0||||||||20131001140742
    Q|1|000354|^^^ALL
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "000354", "type": "patient"}],
    }
    assert worklist == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
        H|\\^&|||||||||P||20100323
        Q|1|^1234567||ALL||||||||O
        L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&||||||||||P||20130715141912
    L|1|N
    """
    assert result == expected


def test_decode_result(self):
    date = datetime.datetime(2020, 1, 31, 13, 47, 23)
    worklist = self.decode(
        """
    H|\\^&|||TECAN EVOlution^V 2.0|||||||||20200204143303
    P|1|8402335073
    O|1|8402335073|8402335073|^^^Anti-GAD ELISA^1.000|||||||P
    R|1|^^^Anti-GAD ELISA^1.000^^^Mean conc. (IU/ml)|<Min|IU/ml||||||Pass||tecan|2020-01-31T10:02:21.0644381+01:00|2020-01-31T13:47:23.9213247+01:00|1908004364
    P|2|8408358978
    O|1|8408358978|8408358978|^^^Anti-GAD ELISA^1.000|||||||P
    R|1|^^^Anti-GAD ELISA^1.000^^^Mean conc. (IU/ml)|14.124|IU/ml||||||Pass||tecan|2020-01-31T10:02:21.0644381+01:00|2020-01-31T13:47:23.9213247+01:00|1908004364
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "8402335073",
                "results": [
                    {
                        "code": "Anti-GAD ELISAMean conc. (IU/ml)",
                        "value": "<Min",
                        "dilution": "1.000",
                        "unit": "IU/ml",
                        "operator": "tecan",
                        "instrument": "1908004364",
                        "date": date,
                        "flags": [],
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "8408358978",
                "results": [
                    {
                        "code": "Anti-GAD ELISAMean conc. (IU/ml)",
                        "value": "14.124",
                        "dilution": "1.000",
                        "unit": "IU/ml",
                        "operator": "tecan",
                        "instrument": "1908004364",
                        "date": date,
                        "flags": [],
                    }
                ],
            },
        ],
    }
    assert worklist == expected
